## model assisted estimate of population average
## version 5: add user friendly interface

test <- function()
{
  rm(list=ls())
  load(file="wds.rda")
  ## define variables
  wds$STRATUM2 <- as.numeric(wds$stratum==2) ## define stratum variable
  wds$STRATUM3 <- as.numeric(wds$stratum==3)
  wds$t_POINT_Y <- scale(wds$POINT_Y) ## scale trend for numerical stability
  wds$t_DEPTH_B <- scale(wds$DEPTH_B) ## scale bottom detph for numerical stability
  l <- split(wds,wds$YEAR_)
  # load grid definition from Chesapeake Bay Program
  grid <- read.csv(file="cbp_grid_090716.csv",head=T)
  grid$STRATUM2 <- as.numeric(grid$STRATUM==2) ## define stratum variable
  grid$STRATUM3 <- as.numeric(grid$STRATUM==3)
  grid$t_POINT_Y <- scale(grid$POINT_Y) ## scale trend for numerical stability
  grid$t_DEPTH_B <- scale(grid$DEPTH_B) ## scale bottom detph for numerical stability
  grid$Dummy <- 1 ## no segment, bay-wide estimation
  ## load results
  load(file="~/Spring2017/Projects/Winter_Dredge_Survey/inla_2/Storage_3k/wds070716_y.rda")
  
  load(file="design08292015.rda")
  
  source("ma_el_5.R")
  samples.zipbk.ms <- vector("list",length(l))
  for(i in 1:length(l)){
    cat("processing ",i,"\n")
    samples.zipbk.ms[[i]] <- ma.el(
      x=l[[i]],
      grid=grid,
      y=r[[i]]$predSample,
      formula=y~POINT_X+POINT_Y,
      gcoord=NULL,
      FUN=median
    )
  }
  tmp <- cbind(design,do.call(rbind,samples.zipbk.ms))
  
  png(filename = "WDS/est.png",width = 1000,heigh=1000,res=200)
  oldpar <- par(pty="s",mfrow=c(1,2))
  zl <- range(c(tmp[,"xbar"],tmp[,"el.fit.el"],tmp[,"el.mcmc"]))
  plot(tmp[,"xbar"],tmp[,"el.fit.el"],main="Calibration based on INLA",
       xlab="design",ylab="calibration",xlim=zl,ylim=zl)
  abline(0,1,lwd=2,lty=2,col="blue")
  plot(tmp[,"xbar"],tmp[,"el.mcmc"],main="Calibration based on INLA",
       xlab="design",ylab="MCMC calibration",xlim=zl,ylim=zl)
  abline(0,1,lwd=2,lty=2,col="blue")
  par(oldpar)
  dev.off()
}
require(FNN)

ma.el <- function(x,grid,y,formula,gcoord=NULL,FUN=median,gamma=0.90,cv.max=10,verbose=TRUE)
{
  ## x : sample data
  ## grid: sampling frame 
  ## y  : auxiliary data on the frame (if a vector - then point calibration)
  ##               if a matrix - then posterior calibration
  ## formula: a formula defining data and geographic coordinate in x
  ## gcoord: a formula defining geographic coordinate in grid
  ## FUN   : for posterior samples of y, the function of point estimate
  
  ## gamma : for posterior sample based calibration, the percentile of posterior
  ## coefficient of variation to exclude
  ## cv.max : threshold of coefficient of variation to exclude
  
  ## value: classical estimates plus EL-based calibration results 
  ##  mean and standard error based on classical design and
  ##  mean and standard error based on EL calibrated to posterior summary
  ##  mean, standard error due to model, sampling and total calibrated
  ##  to full posterior samples
  
  stopifnot(length(as.character(formula))==3)
  ## formula must be of the format data ~ xcoord + ycoord
  
  ## define the x-y coordinates on the sampling frame
  if(is.null(gcoord)){
    ## default to the same as the sample frame
    l.txt <- as.character(formula)
    gcoord <- as.formula(paste(l.txt[1],l.txt[3],sep=""))
  }
  else{
    stopifnot(length(as.character(gcoord))==2)
    ## gcoord must be of the formualt [bland] ~ xcoord + ycoord
  }
  
  
  ## remove missing values
  l.data <- na.omit(x[,all.vars(formula)])
  l.grid <- na.omit(grid[,all.vars(gcoord)])

  ## check calibration data dimensions
  if(is.null(dim(y))){
    if(!is.null(attr(l.grid,"na.action"))){
      l.na.action <- as.numeric(attr(l.grid,"na.action"))
      y <- y[-l.na.action]
    }
    ## a vector assumed posterior point estimates
    stopifnot(length(y)==nrow(l.grid))
    ## must of the dimension of the sampling frame
    fit.cpue <- y
  }
  else{
    stopifnot(length(dim(y))==2)
    if(!is.null(attr(l.grid,"na.action"))){
      l.na.action <- as.numeric(attr(l.grid,"na.action"))
      y <- y[-l.na.action,]
    }
    stopifnot(nrow(y)==nrow(l.grid))
    #fit.cpue <- apply(y,1,FUN)
    fit.cpue <- apply(y,1,FUN,na.rm=TRUE)
  }
  
  ## observed cpue
  obs.cpue <- l.data[,all.vars(formula)[1]]
  ## data mapping
  sampled.grid <- get.knnx(
    query=as.matrix(l.data[,all.vars(formula)[2:3]]),
    data=as.matrix(l.grid[,all.vars(gcoord)]),
    k=1)$nn.index[,1]
  
  ## inclusion probability
  pi.lst <- ma.el.prop(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue)

  ## model calibration based estimate
  r.calib <- ma.el.work(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue,pi.lst=pi.lst)

  ## storage
  r <- c(xbar=mean(obs.cpue),se=sd(obs.cpue)/sqrt(length(obs.cpue)),
         el= r.calib$value[1],
         el.pt.se= r.calib$value[2],
         el.mcmc=NA,
         el.mcmc.model.se=NA,
         el.mcmc.pi.se=NA,
         el.mcmc.se=NA)
  
  if(!is.null(dim(y))){
    ## posterior samples
    ## remove missing values from the posterior sampling
    ii <- apply(y,2,function(x) any(is.na(x)))
    y.complete <- y[,!ii]
    ## remove posterior draws with extreme variability
    ## due to INLA instabilities
    if(verbose){
      cat("removing ",sum(ii)," posterior draws with missing values.\n")
    }
    
    ## posterior cv
    cv.est <- apply(y.complete,2,function(x){
      sd(x)/mean(x)
    })
    ## limit to consider as extremes
    cv.limit <- min(quantile(cv.est,probs=gamma),cv.max)
    ##
    y.valid <- y.complete[,cv.est<=cv.limit]
    
    if(verbose){
      cat("removing",sum(cv.est>cv.limit)," posterior draws with extreme posterior samples.\n")
      cat("using",ncol(y.valid)," posterior samples.\n")
    }
    ## apply calibration to the "valid" samples
    out <- apply(y.valid,2,function(x){
      ma.el.work(y=obs.cpue,idx=sampled.grid,Yhat=x,pi.lst=pi.lst)$value
    })
    
    (fit <- mean(out[1,])) ## calibrated mean
    (sigmasq <- var(out[1,]) + mean(out[2,]^2)) ## calibrated standard error
    ## calibration results
    r.calib.2 <- c(fit,sd(out[1,]),sqrt(mean(out[2,]^2)),sqrt(sigmasq))
    r[5:8] <- r.calib.2
  }
  
  ## return
  r
  
}
## helper function to compute inclusion probability
ma.el.prop <- function(y,idx,Yhat,w=NULL)
{
  ## calculate design weights (assuming SRSwr)
  n <- length(y)
  N <- length(Yhat)
  if(is.null(w)){
    ## first order inclusion
    di <- 1-(1-1/N)^n
    pi <- rep(di,n)
  }
  else{
    pi <- 1-(1-w)^n
  }
  ## second order inclusion
  tmp <- outer(pi,pi,FUN="+")
  if(is.null(w)){
    pi2 <- tmp - (1 - (1-2/N)^n)
  }
  else{
    pi2.neither <- (1-outer(w,w,FUN="+"))^n ## probability of neither i nor j in sample
    pi2.either <- 1 - pi2.neither  ## probability of either i or j in sample
    ## probability of both i and j in sample
    pi2 <- tmp - pi2.either  ## general probability addition rule
  }
  list(pi=pi,pij=pi2)
}

ma.el.work <- function(y,idx,Yhat,alpha=0.05,w=NULL,pi.lst=NULL){
  ## y: sampled data y
  ## idx: sample indices
  ## Yhat: model prediction at all units - NO missing values!
  ## w: probability proportional to each item being sampled

  ## calculate design weights (assuming SRSwr)
  n <- length(y)
  N <- length(Yhat)
  
  if(is.null(pi.lst)){
    pi.lst <- ma.el.prop(y=y,idx=idx,Yhat=Yhat,w=w)
  }
  pi <- pi.lst$pi
  pi2 <- pi.lst$pij
  
  ## model calibration
  ldata <- data.frame(yo=y,xo=Yhat[idx])
  tmp <- lm(yo~xo,data=ldata)
  yhat <- predict(tmp) ## calibration at observed sample
  yhat.all <- predict(tmp,list(xo=Yhat)) ## calibration at all covariances
  
  ## derive EL weights
  ds <- 1/pi
  dstar <- ds/sum(ds) ## standardized design weights
  aux <- mean(yhat.all,na.rm=TRUE)
  lambda <- Lag1(u=yhat,ds=dstar,mu=aux)
  weight.el <- (dstar)/(1+ lambda * (yhat-aux))
  
  
  ## estimate
  fit <- sum(weight.el * y)
  
  ## design based variance estimate
  e <- y - yhat
  part1 <-  sum((1-pi)*(e/pi)^2)
  delta <- 1-outer(pi,pi)/pi2
  emat <- outer(e/pi,e/pi)
  tmp2 <- delta * emat
  diag(tmp2) <- 0
  part2 <- sum(tmp2)
  ssq <-  (part1 + part2)/N^2

  if(ssq<0) warning("negative variance estimates\n")
  crit <- qnorm(1-alpha/2)
  r <- c(fit.el=fit,se.el=sqrt(ssq),lwr.el=fit-crit*sqrt(ssq),
         upr.el=fit+crit*sqrt(ssq))
  list(value=r)
}

debug <- function()
{
  rm(list=ls())
  load(file="Scratch/debug.rda")
  source("ma_el_5b.R") 
  r <- ma.el.work(y=obs.cpue,idx=sampled.grid,Yhat=y[,7],pi.lst=pi.lst)
  
}
## Algorithm 2
## Wu 2005 Survey Methodology Vol. 31 No 2 pp239-243 2005

## 09/07/2018
## When the algorithm can not be applied return missing values
Lag1 <- function(u,ds,mu)
{
  ## u: model calibrated results
  ## ds: design weights
  ## mu: average of population calibrated results
  ## return : lagrange multiplier lambda
  
  L <- -1/max(u-mu)
  R <- -1/min(u-mu)
  if(is.na(L)||is.na(R)){
    return(NA)
  }
  if(L>=R){
    return(NA)
  }
  dif <- 1
  tol <-  1e-08
  
  # browser()
  # g <- function(lambda) sum(ds*(u-mu)/(1+lambda*(u-mu)))
  # lseq <- seq(-0.0001,0.0001,len=999)
  # oseq <- sapply(lseq,g)
  # plot(lseq,oseq,type="l")
  
  while(dif>tol){
    M <- (L+R)/2
    glam <- sum((ds*(u-mu))/(1+M*(u-mu)))
    if(glam>0) L <- M
    if(glam<0) R <- M
    dif <- abs(glam)
  }
  return(M)
}

Lag2<-function(u,ds,mu)
{
  n <- length(ds)
  u <- u-rep(1,n)%*%t(mu) 
  M <- 0*mu
  dif <- 1
  tol <- 1e-08 
  while(dif > tol){
    D1 <- 0*mu
    DD <- D1 %*% t(D1)
    for(i in 1:n){
      aa <- as.numeric(1+t(M)%*%u[i,]) 
      D1 <- D1+ds[i]*u[i,]/aa
      DD <- DD-ds[i]*(u[i,]%*%t(u[i,]))/aa^2
    }
    D2 <- solve(DD,D1,tol=1e-12)
    dif <- max(abs(D2))
    rule <- 1 
    while(rule>0){
      rule <- 0
      if(min(1+t(M-D2)%*%t(u)) <= 0) rule<-rule+1 
      if(rule>0) D2 <- D2/2
    } 
    M <- M-D2
  }
  return(M)
}


## sample code for stratified sampling
Lag3 <- function(n,x,ds,X,W)
{
  ## n: Hx1 vector of stratum sample sizes
  ## x: sum(n) x p data matrix ordered by strata
  ## ds: sum(n) x 1 vector of normalized initial design weights for stratum h
  ## X: p x 1 vector of population means
  ## W: Hx1 vector of stratum weights
  ## value:
  ## a list containing
  ## M : solultion to g3(\lambda) = 0
  ## phat: optimal weights \lambda hat
  
  nst<-sum(n)
  k<-length(n)-1
  ntot<-rep(0,k)
  ntot[1]<-n[1]
  for(j in 2:k) ntot[j]<-ntot[j-1]+n[j]
  ist<-matrix(0,nst,k)
  ist[1:n[1],1]<-1
  for(j in 2:k) ist[(ntot[j-1]+1):ntot[j],j]<-1
  uhi<-cbind(ist,x)
  mu<-c(W[1:k],X)
  whi<-rep(W[1],n[1])
  for(j in 2:(k+1)) whi<-c(whi,rep(W[j],n[j]))
  dhi<-whi*ds
  M<-Lag2(uhi,dhi,mu)
  phi<-as.vector(ds/(1+(uhi-rep(1,nst)%*%t(mu))%*%M))
  list(M=M,phat=phi)
}
    