require(INLA)
require(FNN)
## version 15-a: Revision 1
## allow specifying different families for the catch distribution
zipbk.inla <-function(
  query,obs,fixed=~1,random=~s+e,
  family="zeroinflatedpoisson0",
  xcoord='x',ycoord='y',wcoord="w",bcoord='b',zcoord='z',
  max.edge=c(0.15,1),cutoff=c(0.1,0.5),boundary=NULL,mesh=NULL,
  prior.variance.nominal=1,prior.range.nominal=NULL,theta.prior.mean=c(0,0),theta.prior.prec=c(0.1,1),
  unstruct.hyper='list(prec=list(prior="logtnormal",param=c(0,0.0001)))',
  num.threads=NULL,predict=T,verbose=T,control=list(nsamples=399))
{
  ## query: query location data frame with associated block IDs;
  ## obs : observed location and data frame with values (assumed on point support);
  ## fixed: an INLA formula indicating the fixed effects (or non-spatial effects)
  ## family: INLA likelihood for positive catch distribution
  ## random: a formula indicating the spatial random effects must be ~s+e
  ## xcoord: coordinate for easting /longitude
  ## ycoord: coordinate for northing / latitude
  ## bcoord: coordinate for block in query data frame
  ## wcoord: coordinate for query location weight (e.g. area)
  ## zcoord: coordinate for observation
  
  ## MESH Parameters:
  ## max.edge: maximum edge to define mesh
  ## cutoff: cut off length to define mesh
  ## boundary: boundary polygon to define mesh (of type polygons?)

  ## SPDE Prior Parameters:
  ## prior.variance.nominal: hyper parameter: nominal prior variance for the "SPDE" field
  ## prior.range.nominal: hyper parameter: nominal prior range for the "SPDE" field
  ## theta.prior.mean: hyper parameter: mean for mvnorm prior distribution of "SPDE" sill and range
  ## theta.prior.prec: hyper parameter: precision for independent mvnorm prior distribution of "SPDE" sill and range
  
  ## IID Noise Prior Parameters:
  ## unstruct.hyper: hyper prior for unstructured compoment
  ## alternative 'list(prec=list(prior="logtnormal",param=c(0,0.0001)))'
  ## 'list(theta=list(prior="loggamma",param=c(0.01,0.01)))',
  
  ## Computing Parameters:
  ## num.threads: number of processors for INLA to use
  ## predict: binary scalar whether to generate posterior prediction
  
  ## Values:
  ## the fitted inla model, the approximate posterior samples, and other arguments
  #browser()
  if(verbose) cat("## check family ##\n")
  l_fam <- c("zeroinflatedpoisson0","zeroinflatednbinomial0","gamma")
  if(!(family %in% l_fam)){
    stop("family must be in ",l_fam,"\n")
  }
  if(verbose) cat("## check data ## \n")
  ## 9/6/18 update remove missing values
  l.ii <- attr(na.omit(query[,c(xcoord,ycoord,bcoord,wcoord,all.vars(fixed))]),
               "na.action")
  l.jj <- attr(na.omit(obs[,c(xcoord,ycoord,zcoord,all.vars(fixed))]),
               "na.action")
  if(!is.null(l.ii)){
    query <- query[-l.ii,]
  }
  if(!is.null(l.jj)){
    obs <- obs[-l.jj,]
  }
  
  # potentially check for coordinates, for positive observed values, something else
  l.query <- as.matrix(query[,c(xcoord,ycoord,bcoord,wcoord)])
  l.obs <- as.matrix(obs[,c(xcoord,ycoord,zcoord)])
  
  chk1 <- all(l.obs[,3]>=0)
  if(!chk1){
    stop(zcoord," contains negatives...")
  }
#   chk2 <- mean(l.obs[,3]==0)
#   if(chk2>0){
#     cat(round(chk2,2)," data values are zero, added small constant\n")
#     l.obs[,3] <- l.obs[,3]+epsilon
#   }

  ## detect zero distance
  tmp <- data.frame(l.obs[,1:2])
  coordinates(tmp) <- as.formula(paste("~",xcoord,"+",ycoord,sep=""))
  ii  <- zerodist(tmp)
  if(nrow(ii)>0){
    cat("zero distance detected\n")
    l.obs[unique(ii[,1]),1] <- l.obs[unique(ii[,1]),1]+1000*runif(length(unique(ii[,1])))
    l.obs[unique(ii[,2]),2] <- l.obs[unique(ii[,2]),2]+1000*runif(length(unique(ii[,2])))
  }

  ## responses
  z <- as.numeric(l.obs[,3]>0) ## whether a catch or not
  y <- ifelse(z,l.obs[,3],NA)  ## catch number
  
  ## fixed effect lists
  tmp <- all.vars(fixed)
  if(length(tmp)==0){
    l.dat <- matrix(1,nrow(obs)+nrow(query),1)
  }
  else{
    l.dat <- rbind(obs[,tmp],query[,tmp])
  } 
  z.obs.lst <- zipbk.inla.fixed(obs,fixed,prefix="z") ## observed zeros
  z.query.lst <- zipbk.inla.fixed(l.dat,fixed,prefix="z") ## query zeros
  y.obs.lst <- zipbk.inla.fixed(obs,fixed,prefix="y") ## observed numbers
  y.query.lst <- zipbk.inla.fixed(l.dat,fixed,prefix="y") ## query numbers
  fixed.noint <- zipbk.inla.formula(l.dat,fixed) ## update formula to without intercept when factors are in fixed
 
  ## random effects
  l.terms <- zipbk.inla.random(formula=random)
  
  cat("## fit a SPDE model to data ## \n")
  #browser()
  ## create a MESH
  coords <- l.obs[,1:2]
  if(is.null(boundary)){
    bnd <- inla.nonconvex.hull(coords)
  }
  else{
    bnd <- boundary
  }
  #browser()
  if(is.null(mesh)){
    mesh  <- inla.mesh.2d(coords,boundary=bnd,max.edge=max.edge,cutoff=cutoff)
  }
  
  ## define SPDE prior
  size <- max(c(diff(range(mesh$loc[, 1])), diff(range(mesh$loc[, 2])),diff(range(mesh$loc[,3]))))
  if(is.null(prior.range.nominal)){
    prior.range.nominal <- size/5
  }
  kappa0 <- sqrt(8)/prior.range.nominal
  tau0 <- 1/(sqrt(4*pi)*kappa0*sqrt(prior.variance.nominal))
  spde <- inla.spde2.matern(mesh, alpha=2,B.tau=cbind(log(tau0),-1,+1),
                            B.kappa=cbind(log(kappa0),0,-1),
                            theta.prior.mean=theta.prior.mean,
                            theta.prior.prec=theta.prior.prec)

  ## Define INLA data structure
  A <- inla.spde.make.A(mesh, loc=coords)
  stk.z <- inla.stack(tag="catch",
                      data=list(y=cbind(z,NA)),
                      A=list(A,1),
                      effects=list(
                        list(field=1:spde$n.spde),
                        c(list(unstruct=1:length(z)),z.obs.lst)
                        )
                      )
  stk.y <- inla.stack(tag="num",
                      data=list(y=cbind(NA,y)),
                      A=list(A,1),
                      effects=list(
                        list(field.copy=1:spde$n.spde),
                        c(list(unstruct.copy=1:length(y)),y.obs.lst)
                        )
                      )
  stk.zy <- inla.stack(stk.z,stk.y)

  ## Define INLA model
  fixed.vars <- c("0",names(z.obs.lst),names(y.obs.lst))
  spatial.vars <- NULL
  if(l.terms$is.spde){
    spatial.vars <- c(spatial.vars,
                      c("f(field,model=spde)",'f(field.copy,copy="field",fixed=FALSE)'))
  }
  if(l.terms$is.overdisp){
    if(is.null(unstruct.hyper)){
      spatial.vars <- c(
        spatial.vars,c(
          'f(unstruct,model="iid")',
          'f(unstruct.copy,copy="unstruct",fixed=FALSE)'
        )
      )
    }
    else{
      spatial.vars <- c(
        spatial.vars,c(
          paste('f(unstruct,model="iid",hyper=',unstruct.hyper,')',sep=""),
          'f(unstruct.copy,copy="unstruct",fixed=FALSE)'
        )
      )
    }
  }
  if(verbose) cat(spatial.vars,"\n")
  # spatial.vars <- c("f(field,model=spde)",'f(field.copy,copy="field",fixed=FALSE)', 
  #                   'f(unstruct,model="iid",hyper=',unstruct.hyper,')',
  #                   'f(unstruct.copy,copy="unstruct",fixed=FALSE)')
  formula <- reformulate(c(fixed.vars,spatial.vars),response="y")
#   formula <- y~ 0 + z.b0+y.b0+f(field,model=spde)+f(field.copy,copy="field",fixed=FALSE)+ 
#     f(unstruct,model="iid",hyper=list(theta=list(prior="loggamma",param=unstruct.parm)))+
#     f(unstruct.copy,copy="unstruct",fixed=FALSE)

  #browser()
  if(verbose) cat("## extrapolate to study area ## \n")
  pcoord <- rbind(coords,l.query[,1:2])
  Afull <- inla.spde.make.A(mesh,loc=pcoord)
  zfull <- c(z,rep(NA,nrow(l.query)))
  yfull <- c(y,rep(NA,nrow(l.query)))
  stk.zf <- inla.stack(tag="catch.pred",
                       data=list(y=cbind(zfull,NA)),
                       A=list(Afull,1),
                       effects=list(
                         list(field=1:spde$n.spde),
                         c(list(unstruct=1:length(zfull)),z.query.lst)
                         )
                       )
  stk.yf <- inla.stack(tag="num.pred",
                       data=list(y=cbind(NA,yfull)),
                       A=list(Afull,1),
                       effects=list(
                         list(field.copy=1:spde$n.spde),
                         c(list(unstruct.copy=1:length(yfull)),y.query.lst)
                         )
                       )
  stk.zyf <- inla.stack(stk.zf,stk.yf)
  
  #browser()
  ## mapping to predictive distribution
  z.index <- inla.stack.index(stk.zyf,"catch.pred")$data
  y.index <- inla.stack.index(stk.zyf,"num.pred")$data

  ## use full predictive inference
  if(family %in% c('zeroinflatedpoisson0','zeroinflatednbinomial0')){
    mod1 <- try(inla(formula,family=c('binomial',family),
                     data=inla.stack.data(stk.zyf),
                     control.compute=list(config=TRUE,dic=TRUE),
                     control.family=list(
                       list(),
                       list(hyper=list(prob=list(initial=-7,fixed=TRUE)))),
                     control.predictor=list(A=inla.stack.A(stk.zyf),compute=TRUE),
                     control.inla=list(int.strategy="eb",reordering ="metis"),
                     num.threads=num.threads,
                     control.results=list(return.marginals.predictor=FALSE),
                     control.fixed=list(expand.factor.strategy="inla"),
                     verbose=verbose
    ))    
    
  }
  if(family == "gamma"){
    mod1 <- try(inla(formula,family=c('binomial','gamma'),
                     data=inla.stack.data(stk.zyf),
                     control.compute=list(config=TRUE,dic=TRUE),
                     control.predictor=list(A=inla.stack.A(stk.zyf),compute=TRUE),
                     control.inla=list(int.strategy="eb"),
                     num.threads=num.threads,
                     control.results=list(return.marginals.predictor=FALSE),
                     control.fixed=list(expand.factor.strategy="inla"),
                     verbose=verbose
    ))    
    
  }
 
  if(class(mod1)=="try-error"){
    return(list(est=NULL,inla=NULL,samples=NULL,predict=predict,
                fixed.2=fixed.noint,terms=l.terms,predSample=NULL,dic=NULL))
  }
  
  #browser()
  ## parameter estimation
  
  tausq <- rep(NA,3)
  sigmasq <- h <- rep(NA,3)
  
  if(l.terms$is.spde){
    r.f <- try(inla.spde2.result(mod1, "field", spde, do.transf=TRUE),silent=TRUE)
    if(class(r.f)!="try-error"){
      ## partial sill
      sigmasq <- inla.qmarginal(c(0.5,0.025,0.975),r.f$marginals.variance.nominal[[1]])
      ## range
      h <- inla.qmarginal(c(0.5,0.025,0.975),r.f$marginals.range.nominal[[1]])
    }
  }

  if(l.terms$is.overdisp){
    ## nugget
    #tmp <- try(inla.tmarginal(function(x) 1/(x), mod1$marginals.hy[["Precision for unstruct"]]))
    tmp <- mod1$summary.hyperpar["Precision for unstruct",3:5]
    #if(class(tmp)!="try-error"){
    #  tausq <- inla.qmarginal(c(0.5,0.025,0.975),tmp)
    #}
    tausq <- 1/as.matrix(tmp)[c(2,3,1)]
  }
  
  ## start location for beta parameters
  hyper.loc <- 0 
  if(family=="zeroinflatednbinomial0"){
    hyper.loc <- 1
  }
  if(family=="gamma"){
    hyper.loc <- 1
  }
  alpha <- as.matrix(mod1$summary.fixed[,c(4,3,5)])
  betav <- matrix(NA,2,3)
  if(l.terms$is.spde & l.terms$is.overdisp){
    betav <- as.matrix(mod1$summary.hyperpar[hyper.loc+4:5,c(4,3,5)])
  }
  if(l.terms$is.spde & !l.terms$is.overdisp){
    betav[1,] <- unlist(mod1$summary.hyperpar[hyper.loc+3,c(4,3,5),drop=TRUE])
  }
  if(!l.terms$is.spde & l.terms$is.overdisp){
    betav[2,] <- unlist(mod1$summary.hyperpar[hyper.loc+2,c(4,3,5),drop=TRUE])
  }

  parm <- as.matrix(rbind(alpha,betav,sigmasq,h,tausq))
  colnames(parm) <- c("fit","lwr","upr")
  rownames(parm) <- c(row.names(alpha),"beta.field","beta.unstruct","sigmasq","range","tausq")


  predSample <- NULL

  if(control$nsamples>0){
    cat("## approximate posterior samples ## \n")
    ## posterior samples
    samples <- inla.posterior.sample(control$nsamples,mod1)
  }
  else{
    samples <- NULL
  }
  
  browser()
  
  if(predict & !is.null(samples)){ ## generate predictive samples
    #l_fam <- c("zeroinflatedpoisson0","zeroinflatednbinomial0","gamma")
    if(family == "zeroinflatedpoisson0"){
      predSample <- sapply(samples,zipbk.inla.pred.poisson.3,
                           zindex=z.index,yindex=y.index,
                           n=nrow(l.obs),m=nrow(l.query))
    }
    if(family == "zeroinflatednbinomial0"){
      predSample <- sapply(samples,zipbk.inla.pred.nbinom.3,
                           zindex=z.index,yindex=y.index,
                           n=nrow(l.obs),m=nrow(l.query))
      
    }
    if(family == "gamma"){
      predSample <- sapply(samples,zipbk.inla.pred.gamma.3,
                           zindex=z.index,yindex=y.index,
                           n=nrow(l.obs),m=nrow(l.query))
      
    }
    ## the same predictive distribution applies for all random structure
    
    # if(l.terms$is.spde & l.terms$is.overdisp){ ## both spde and over-dispersion
    #   predSample <- sapply(samples,zipbk.inla.pred.3,
    #                        zindex=z.index,yindex=y.index,
    #                        n=nrow(l.obs),m=nrow(l.query))
    # }
    # if(!l.terms$is.spde & l.terms$is.overdisp){ ## over-dispersion only
    #   predSample <- sapply(samples,zipbk.inla.pred.3,
    #                        zindex=z.index,yindex=y.index,
    #                        n=nrow(l.obs),m=nrow(l.query))
    # }
    # if(l.terms$is.spde & !l.terms$is.overdisp){ ## spde only
    #   predSample <- sapply(samples,zipbk.inla.pred.3,
    #                        zindex=z.index,yindex=y.index,
    #                        n=nrow(l.obs),m=nrow(l.query))
    # }
    # if(!l.terms$is.spde & !l.terms$is.overdisp){ ## no random effects
    #   predSample <- sapply(samples,zipbk.inla.pred.3,
    #                        zindex=z.index,yindex=y.index,
    #                        n=nrow(l.obs),m=nrow(l.query))
    # }
  }
  #browser()
  l.dic <- with(mod1$dic,c(Dbar=mean.deviance,pD=p.eff,DIC=dic))
  tmp <- with(l.terms,c("SPDE","IID")[c(is.spde,is.overdisp)])
  if(length(tmp)==0) tmp <- ""
  else tmp <- paste(tmp,collapse=" + ")
  l.formula <-  paste(zcoord,"(",family,")~",as.character(fixed)[2],"+",tmp,sep="")
  list(est=parm,inla=mod1,samples=samples,predSample=predSample,predict=predict,fixed.2=fixed.noint,terms=l.terms,dic=l.dic,formula=l.formula)
}

zipbk.inla.pred.poisson.3 <- function(l,zindex,yindex,n,m){
  ## helper function to derive posterior predictive distribution
  ## l : list of object from INLA approximate posterior sample
  ## zindex : index of catch process
  ## yindex : index of number of catch given catch 
  ## family : inla family to sample posterior predictive distribution
  ## value: a posterior sample draw from m prediction grid
  
  names <- rownames(l$latent)

  eta.z <- l$latent[zindex,1]
  eta.y <- l$latent[yindex,1]
  
  #browser()
  
  ## posterior predictive distribution
  #p <- binomial()$linkinv(eta.z)   ## posterior probability of catch
  p <- inla.link.logit(eta.z,inverse=TRUE)
  #lambda <- poisson()$linkinv(eta.y) 
  #l_fam <- c("zeroinflatedpoisson0","zeroinflatednbinomial0","gamma")
  ## posterior mean of number caught
  lambda <- inla.link.log(eta.y,inverse=TRUE) 

  u <- runif(length(p))
  z <- 0 + (u<p)
  y <- rzi.pois(n=sum(z>0),lambda=lambda[z>0])$sample
  z[z>0] <- y
  zp <- z[n+1:m]
  zp
  
}

zipbk.inla.pred.nbinom.3 <- function(l,zindex,yindex,n,m){
  #browser()
  eta.z <- l$latent[zindex,1]
  eta.y <- l$latent[yindex,1]
  
  ## posterior predictive distribution
  p <- inla.link.logit(eta.z,inverse=TRUE)
  lambda <- inla.link.log(eta.y,inverse=TRUE)
  l.size <- l$hyperpar["size for nbinomial zero-inflated observations[2]"]
  l.prob <- 1/(1+lambda/l.size)
  u <- runif(length(p))
  z <- 0+(u<p)
  y <- rzi.nbinom(n=sum(z>0),size=l.size,prob=l.prob[z>0])$samples
  #print(c(length(y),sum(z>0)))
  z[z>0] <- y
  z[n+1:m]
}

zipbk.inla.pred.gamma.3 <- function(l,zindex,yindex,n,m){
  eta.z <- l$latent[zindex,1]
  eta.y <- l$latent[yindex,1]
  p <- inla.link.logit(eta.z,inverse=TRUE)
  lambda <- inla.link.log(eta.y,inverse=TRUE)
  ## tbd define l.shape and l.rate
  u <- runif(length(p))
  z <- 0+(u<p)
  y <- rgamma(n=sum(z>0),shape=l.shape,rate=l.rate[z>0])
  z[z>0] <- y
  z[n+1:m]
}
zipbk.inla.random <- function(formula){
  l.vars <- all.vars(formula)
  ret <- list(is.spde =FALSE, is.overdisp =FALSE)
  if(length(l.vars)==0){
    return(ret)
  }
  if(any(!(l.vars %in% c("s","e")))){
    stop("random terms must be either s(SPDE) or e(overdisp)\n")
  }
  if(length(l.vars)==1){
    if(l.vars=="s"){
      ret$is.spde <- TRUE
    }
    else{
      ret$is.overdisp <- TRUE
    }
    return(ret)
  }
  if(length(l.vars)==2){
    ret$is.spde <- TRUE
    ret$is.overdisp <- TRUE
  }
  ret
}
zipbk.inla.fixed <- function(dat,formula,prefix="z"){
  ## helper function to generate the lists to fix semi-continous spatial model for zero-inflated catch data
  ## generate the list of data included in the formula with appropriate names
  #browser()
  l.df <- data.frame(tmp=rep(1,nrow(dat)))
  names(l.df) <- paste(prefix,"b0",sep=".")
  l.vars <- all.vars(formula)
  if(length(l.vars)>0){
    tmp <- as.data.frame(dat[,l.vars])
    names(tmp) <- paste(prefix,l.vars,sep=".")
    l.df <- cbind(l.df,tmp)
    
    if("factor" %in% sapply(tmp,class)){
      ## cheap fix for only one factor
      ## would not work with two or more factors...
      l.df <- l.df[,-1]
    }
  }
  as.list(l.df)
}
zipbk.inla.formula <- function(dat,formula){
  ## update the formula to remove intercept if there is a factor
  l.vars <- all.vars(formula)
  if(length(l.vars)>0){
    tmp <- as.data.frame(dat[,l.vars])
    if("factor" %in% sapply(tmp,class)){
      ## cheap fix for only one factor
      ## would not work with two or more factors...
      formula <- as.formula(paste(deparse(formula)," + 0"))
    }
  }
  formula
}
test1 <- function()
{
  rm(list=ls())
  source("zipbk_inla_15a.R")
  source("rzi_2.R")
  source("sim_9.R")
  lst3 <- sim8(ndata=50,nx=20,ny=20,cov=list(numeric=T,factor=T),beta=list(numeric=0.5, factor=c(-1,1)))
  loc <- as.matrix(lst3$obs[,c("x","y")])
  bnd <- inla.nonconvex.hull(loc,0.2,0.2)
  mesh <- inla.mesh.2d(loc,boundary=bnd,offset=c(1,0.5),max.edge=c(0.1,0.5))
  ## cutoff can't be included, otherwise the error
  mesh <- inla.mesh.2d(loc,boundary=bnd,cutoff=c(0.05,0.2),offset=c(1,0.5),max.edge=c(0.1,0.5))
  ## .manifold' does not exist.
  plot(mesh)
  r1 <- with(lst3,zipbk.inla(query,obs,
                             fixed=~X1+F1,random=~s,
                             family="zeroinflatedpoisson0",
                             mesh=mesh,control=list(nsamples=399)))
  r2 <- with(lst3,zipbk.inla(query,obs,
                             fixed=~X1+F1,random=~s+e,
                             family="zeroinflatedpoisson0",
                             mesh=mesh,control=list(nsamples=399)))
  
  r4 <- with(lst3,zipbk.inla(query,obs,
                             fixed=~X1+F1,random=~s,
                             family="zeroinflatednbinomial0",
                             mesh=mesh,control=list(nsamples=399)))
  rbind(r1$dic,r2$dic,r4$dic)
  pred1 <- apply(r1$predSample,1,median)
  pred2 <- apply(r2$predSample,1,median)
  pred4 <- apply(r4$predSample,1,median)
  png("Scratch/zipbk_inla_15_simu.png",width=2000,height=700,res=300)
  oldpar <- par(mfrow=c(1,3),mar=c(3.1,3.1,4.1,1.1))
  plot(lst3$query$z,pred1,xlim=c(0,100),ylim=c(0,100),xlab="simulated",ylab="predicted",
       main="Zero augmented Poisson + SPDE")
  crossprod(lst3$query$z-pred1)
  abline(0,1,col="blue",lwd=2)
  plot(lst3$query$z,pred2,xlim=c(0,100),ylim=c(0,100),xlab="simulated",ylab="predicted",
       main="Zero augmented Poisson + SPDE + IID")
  abline(0,1,col="blue",lwd=2)
  crossprod(lst3$query$z-pred2)
  plot(lst3$query$z,pred4,xlim=c(0,100),ylim=c(0,100),xlab="simulated",ylab="predicted",
       main="Zero augmented Nbinomial")
  abline(0,1,col="blue",lwd=2)
  crossprod(lst3$query$z-pred4)
  par(oldpar)
  dev.off()
}

debug1 <- function()
{
  rm(list=ls())
  load("Scratch/dump.rda")
  source("zipbk_inla_14c.R")
  run.2(elmt=simdat,idx=SRS.idx[,i],bnd=segm.bnd2,
        path=paste("./Scratch/",sep=""),
        txt=paste("B4",Settings$id[j],as.character(i),sep="_"),
        random=~s+e,sigma0=Settings$sigma0[j],prec0=Settings$prec0[j],hyper0=Settings$hyper0[j]
  )
  ## not able to duplicate errors on PC, try on cluster
  ## it seems the error is related to heavily skewed precision estimates
  ## of the unstructured component
  ## such that nugget calculation results in numerical issue.
  ## should have been fixed by try the evaluation first
  ## as the main goal is predictive in this case.
  ## in the future, may try a directly inversion of the quantiles
  
}

debug2 <- function()
{
  ## error run long time...
  rm(list=ls())
  load("Scratch/dump2.rda")
  source("zipbk_inla_14c.R")
  run.2 <- function(elmt,idx,bnd,path,txt,random=~s+e,sigma0=0.5,prec0=0.1,hyper0='list(theta=list(prior="loggamma",param=c(0.01,0.01)))')
  {
    ## elmt: a data frame object containing simulated data
    ## idx : index of simple random sample
    ## bnd : boundary to build INLA mesh (inla.mesh.segment object)
    ## path : storage path
    ## txt  : storage root text 01/06/12
    obs <- elmt[idx,]
    elmt$Dummy <- 1
    r <- zipbk.inla(query=elmt,obs=obs,fixed=~POINT_Y+DEPTH_B,random=random,
                    xcoord='POINT_X',ycoord='POINT_Y',wcoord="AREA",bcoord='Dummy',zcoord="y", 
                    max.edge=c(5000,1000000),cutoff=1000,boundary=bnd,mesh=NULL,
                    prior.variance.nominal=sigma0,prior.range.nominal=NULL,theta.prior.mean=c(0,0), theta.prior.prec=c(0.1,prec0),
                    unstruct.hyper= hyper0,
                    num.threads=1,predict=T,verbose=F,control=list(nsamples=399))
    r$inla <- NULL
    r$samples <- NULL
    
    ## incorporate the MCMC calibration analysis here...
    source("Scratch/ma_el_4.R")
    ## observed cpue
    obs.cpue <- obs[,"y"]
    ## data mapping
    require(FNN)
    sampled.grid <- get.knnx(
      query=as.matrix(obs[,c("POINT_X","POINT_Y")]),
      data=as.matrix(elmt[,c("POINT_X","POINT_Y")]),
      k=1)$nn.index[,1]
    fit.cpue <- apply(r$predSample,1,median)
    ## naive inclusion probability
    pi.lst <- ma.el.prop(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue)
    
    cat("model calibration..")
    #browser()
    ## model calibration based estimate
    r.calib <- ma.el(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue,pi.lst=pi.lst)
    cat("done\n")
    cat("model calibration and MCMC...")
    out <- apply(r$predSample,2,function(x){
      r <- ma.el(y=obs.cpue,idx=sampled.grid,Yhat=x,pi.lst=pi.lst)
      r$value
    })
    cat("done\n")
    (fit <- mean(out[1,]))
    mean.mcmc <- apply(r$predSample,2,mean)
    cat("var(Y.bar)=",var(mean.mcmc),",var(y.bar.se)=",var(out[1,]),"\n")
    #(sigmasq <- var(out[1,]) + mean(out[2,]^2))
    (sigmasq <- var(mean.mcmc) + mean(out[2,]^2))
    r$value <- c(mean(obs.cpue),sd(obs.cpue)/sqrt(length(obs.cpue)),
                 r.calib$value[1:2],
                 fit,sd(mean.mcmc),sqrt(mean(out[2,]^2)),sqrt(sigmasq))
    #browser()
    r$ma.out <- out
    r1 <- r[c("predSample","value","total.ms","total.mb","ma.out")]
    save(r1,file=paste(path,"/r",gsub("\\.","_",txt),".rda",sep=""))
    r[c("value","total.ms","total.mb")]
  }
  
  ## SPDE only, Default Hyper-prior
  run.2(elmt=simdat,idx=SRS.idx[,i],bnd=segm.bnd2,
        path=paste("./Revision1_",args[1],"/",sep=""),
        txt=paste("B3",Settings$id[j],as.character(i),sep="_"),
        random=~s,sigma0=Settings$sigma0[j],prec0=Settings$prec0[j],hyper0=Settings$hyper0[j]
        )
  ## error related to predictive distribution over-predicting, making predicted
  ## population mean no longer an interior point of the sampled data
  ## hence EL estimate no longer exists.
  
  ## now return NAs in these scenarioes
}

debug3 <- function(){
  rm(list=ls())
  setwd("C:/Users/dliang/Google Drive/Spring_2017/Winter_Dredge_Survey/inla_2")
  load("Scratch/dump3.rda")
  source("zipbk_inla_15a.R")
  source("rzi_2.R")
  elmt$Dummy <- 1
  obs <- elmt[idx,]
  r <- zipbk.inla(query=elmt,obs=obs,fixed=~POINT_Y+DEPTH_B,random=~s,
                  family="zeroinflatednbinomial0",
                  xcoord='POINT_X',ycoord='POINT_Y',wcoord="AREA",bcoord='Dummy',zcoord="y", 
                  max.edge=c(5000,1000000),cutoff=1000,boundary=bnd,mesh=NULL,
                  prior.variance.nominal=1,prior.range.nominal=NULL,
                  theta.prior.mean=c(0,0), theta.prior.prec=c(0.1,1),
                  unstruct.hyper= 'list(theta=list(prior="loggamma",param=c(1,1)))',
                  num.threads=NULL,predict=T,verbose=T,control=list(nsamples=399))
  r$inla <- NULL
  r$samples <- NULL
  
  ## incorporate the MCMC calibration analysis here...
  source("Scratch/ma_el_4.R")
  ## observed cpue
  obs.cpue <- obs[,"y"]
  ## data mapping
  require(FNN)
  sampled.grid <- get.knnx(
    query=as.matrix(obs[,c("POINT_X","POINT_Y")]),
    data=as.matrix(elmt[,c("POINT_X","POINT_Y")]),
    k=1)$nn.index[,1]
  fit.cpue <- apply(r$predSample,1,median)
  ## naive inclusion probability
  pi.lst <- ma.el.prop(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue)
  
  cat("model calibration..")
  #browser()
  ## model calibration based estimate
  r.calib <- ma.el(y=obs.cpue,idx=sampled.grid,Yhat=fit.cpue,pi.lst=pi.lst)
  cat("done\n")
  cat("model calibration and MCMC...")
  out <- apply(r$predSample,2,function(x){
    r <- ma.el(y=obs.cpue,idx=sampled.grid,Yhat=x,pi.lst=pi.lst)
    r$value
  })
  cat("done\n")
  (fit <- mean(out[1,]))
  mean.mcmc <- apply(r$predSample,2,mean)
  cat("var(Y.bar)=",var(mean.mcmc),",var(y.bar.se)=",var(out[1,]),"\n")
  #(sigmasq <- var(out[1,]) + mean(out[2,]^2))
  (sigmasq <- var(mean.mcmc) + mean(out[2,]^2))
  c(mean(obs.cpue),sd(obs.cpue)/sqrt(length(obs.cpue)),
     r.calib$value[1:2],
     fit,sd(mean.mcmc),sqrt(mean(out[2,]^2)),sqrt(sigmasq))
}